/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.util.Fractal;
import org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.extension.PetalsExtensionController;
import org.ow2.petals.microkernel.api.extension.PetalsPostExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * Controller of the Petals post-extension 'Web-Service API'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ExtensionController implements PetalsPostExtensionController {

    /**
     * Name of the property enabling/disabling the WS API.
     */
    public static final String PROPERTY_NAME_ENABLING_WSAPI = "petals.ws-api";

    /**
     * Default value of the property {@value #WSAPI}.
     */
    private static final boolean DEFAULT_WSAPI = true;

    /**
     * The name of the resources containing the Fractal composite definition
     * associated to the Petals ESB WS API
     */
    private static final String WSAPI_COMPOSITE = "WsApi";

    /**
     * The name set to the Fractal composite associated to the Petals ESB WS API
     */
    private static final String WSAPI_COMPONENT_NAME = "WsApi";

    /**
     * The name of the extension for logging purposes.
     */
    private final static String EXTENSION_NAME = "Petals ESB Web-Service API";

    /**
     * The name of the client interface of the configuration service of the
     * Petals ESB WS API composite
     */
    private final static String CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * The name of the client interface of the atomic deployment service of the
     * Petals ESB WS API composite
     */
    private final static String ATOMICDEPLOYMENT_FRACTAL_ITF_NAME = "atomicDeploymentService";

    /**
     * The name of the client interface of the admin service of the Petals ESB
     * WS API composite
     */
    private final static String ADMIN_FRACTAL_ITF_NAME = "adminService";

    /**
     * The name of the client interface of the container service of the Petals ESB WS API composite
     */
    private final static String CONTAINER_FRACTAL_ITF_NAME = "containerService";

    /**
     * The name of the client interface of the Petals admin service of the Petals ESB WS API composite
     */
    private final static String PETALSADMIN_FRACTAL_ITF_NAME = "petalsAdminService";

    /**
     * The name of the client interface of the topology service of the Petals
     * ESB WS API composite
     */
    private final static String TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * The name of the client interface of the registry service of the Petals
     * ESB WS API composite
     */
    private final static String REGISTRY_FRACTAL_ITF_NAME = "registry";

    /**
     * The name of the client interface of the component installation service of
     * the Petals ESB WS API composite
     */
    private final static String COMPONENTINSTALLATION_FRACTAL_ITF_NAME = "componentInstallationService";

    /**
     * The name of the client interface of the component installation service of
     * the Petals ESB WS API composite
     */
    private final static String ARTIFACTREPOSITORY_FRACTAL_ITF_NAME = "artifactrepository";

    /**
     * {@inheritDoc}
     */
    public Component createFractalComponent(final Component extensionComposite)
            throws CreationExtensionException {

        try {
            // Create the Fractal component associated to the Petals ESB
            // WS API
            final Component extensionImpl = FractalHelper.createCompositeComponent(WSAPI_COMPOSITE);

            // Add the Petals ESB WS API component in the Extension
            // composite
            FractalHelper.addComponent(extensionImpl, extensionComposite, null, WSAPI_COMPONENT_NAME);

            // Next we bind client interfaces of the Petals ESB Artifact
            // Repository to the right interfaces
            final ContentController extensionCompositeCC = Fractal.getContentController(extensionComposite);

            final BindingController extensionBC = Fractal.getBindingController(extensionImpl);

            // As the configuration component is not a sub-component of the
            // extension composite, we link the client interface "configuration"
            // of the WS API to the client interface "configuration" of the
            // extension composite.
            final Interface configurationServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.CONFIGURATION_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            // As the atomic deployment component is not a sub-component of the
            // extension composite, we link the client interface
            // "atomicDeploymentService" of the WS API to the client interface
            // "atomicDeployment" of the extension composite.
            final Interface atomicDeploymentServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.ATOMICDEPLOYMENT_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.ATOMICDEPLOYMENT_FRACTAL_ITF_NAME,
                    atomicDeploymentServiceItf);

            // As the admin component is not a sub-component of the extension
            // composite, we link the client interface "adminService" of the WS
            // API to the client interface "admin" of the extension composite.
            final Interface adminServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.ADMIN_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.ADMIN_FRACTAL_ITF_NAME, adminServiceItf);

            // As the container composite is not a sub-component of the extension
            // composite, we link the client interface "containerService" of the WS
            // API to the client interface "containerService" of the extension composite.
            final Interface containerServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.CONTAINER_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.CONTAINER_FRACTAL_ITF_NAME, containerServiceItf);

            // As the Petals admin component is not a sub-component of the
            // extension composite, we link the client interface
            // "petalsAdminService" of the WS API to the client interface
            // "petalsAdmin" of the extension composite.
            final Interface petalsAdminServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.PETALSADMIN_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.PETALSADMIN_FRACTAL_ITF_NAME,
                    petalsAdminServiceItf);

            // As the topology component is not a sub-component of the extension
            // composite, we link the client interface "topology" of the WS
            // API to the client interface "topology" of the extension
            // composite.
            final Interface topologyServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.TOPOLOGY_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.TOPOLOGY_FRACTAL_ITF_NAME, topologyServiceItf);

            // As the Petals admin component is not a sub-component of the
            // extension composite, we link the client interface "registry" of
            // the WS API to the client interface "registry" of the extension
            // composite.
            final Interface registryServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.REGISTRY_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.REGISTRY_FRACTAL_ITF_NAME, registryServiceItf);

            // As the component installation component is not a sub-component of
            // the extension composite, we link the client interface
            // "componentInstallationService" of the WS API to the client
            // interface "componentInstallation" of the extension composite.
            final Interface componentInstallationServiceItf = (Interface) extensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.COMPONENTINSTALLATION_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.COMPONENTINSTALLATION_FRACTAL_ITF_NAME,
                    componentInstallationServiceItf);

            // The artifact repository is a sub-component of the extension
            // composite, we retrieve it and link the client interface
            // "artifactRepository" of the WS API to the server interface of the
            // Artifact Repository.
            final Component artifactRepositoryComp = FractalHelper.getComponentByName(extensionCompositeCC,
                    ArtifactRepositoryService.ARTIFACTREPOSITORY_COMPONENT_NAME);
            final Interface artifactRepositoryServiceItf = (Interface) artifactRepositoryComp
                    .getFcInterface(ArtifactRepositoryService.SERVICE_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.ARTIFACTREPOSITORY_FRACTAL_ITF_NAME,
                    artifactRepositoryServiceItf);

            return extensionImpl;
        } catch (final InstantiationException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final NoSuchInterfaceException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final IllegalContentException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final IllegalLifeCycleException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final IllegalBindingException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void removeFractalComponent() {
        // TODO Auto-generated method stub

    }

    /**
     * {@inheritDoc}
     */
    public String getExtensionName() {
        return ExtensionController.EXTENSION_NAME;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isActivated(final ConfigurationService configurationService) throws PetalsExtensionException {
        final String propertyValue = configurationService.getServerProperties().getProperty(
                PROPERTY_NAME_ENABLING_WSAPI);
        final boolean isActivated;
        if (propertyValue != null) {
            isActivated = Boolean.parseBoolean(propertyValue);
        } else {
            isActivated = DEFAULT_WSAPI;
        }

        return isActivated;
    }

    /**
     * {@inheritDoc}
     */
    public String[] getDependencies() {
        return new String[] { ArtifactRepositoryService.EXTENSION_NAME };
    }

    @Override
    public boolean failsPetalsStartup() {
        return false;
    }

}
