/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.cxf.endpoint.Server;
import org.apache.cxf.jaxb.JAXBDataBinding;
import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Cardinality;
import org.objectweb.fractal.fraclet.types.Contingency;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@Component(provides = @Interface(name = "service", signature = WebServiceManager.class))
public class WebServiceManagerImpl implements WebServiceManager {

    public static final String WEBSERVICE_FRACTAL_PREFIX = "webservice";

    /**
     * Logger wrapper.
     */
    protected final LoggingUtil log = new LoggingUtil(Logger.getLogger(WebServiceManager.COMPONENT_LOGGER_NAME));

    @Requires(name = "configuration")
    protected ConfigurationService configurationService;

    @Requires(name = WEBSERVICE_FRACTAL_PREFIX, cardinality = Cardinality.COLLECTION, contingency = Contingency.OPTIONAL)
    protected Map<String, KernelWebService> webServices = new HashMap<String, KernelWebService>();

    protected Map<String, Server> servers;

    /**
     * The URL path of the WS API
     */
    private String httpUrlPath;

    /**
     * The listening port of the WS API
     */
    private int httpUrlPort;

    @Lifecycle(step = Step.START)
    public void start() throws RoutingException {
        this.log.start();

        final ContainerConfiguration containerConfig = this.configurationService.getContainerConfiguration();
        final Map<String, String> extraConfProperties = containerConfig.getExtraConfiguration();
        if (extraConfProperties != null) {
            // Listening port
            final String httpUrlPortStr = extraConfProperties.get(PROPERTY_NAME_HTTP_LISTENING_PORT);
            if (httpUrlPortStr != null && !httpUrlPortStr.trim().isEmpty()) {
                try {
                    this.httpUrlPort = Integer.parseInt(httpUrlPortStr);
                    this.log.config("Listening port: " + this.httpUrlPort);
                } catch (final NumberFormatException e) {
                    this.log.warning(String.format(
                            "Invalid value for the listening port: '%s'. Default value used: '%s'", httpUrlPortStr,
                            DEFAULT_HTTP_LISTENING_PORT));
                    this.httpUrlPort = DEFAULT_HTTP_LISTENING_PORT;
                }
            } else {
                this.log.config("The listening port is not defined. Default value used: " + DEFAULT_HTTP_LISTENING_PORT);
                this.httpUrlPort = DEFAULT_HTTP_LISTENING_PORT;
            }

            // URL path
            this.httpUrlPath = extraConfProperties.get(PROPERTY_NAME_HTTP_PATH);
            if (this.httpUrlPath == null || this.httpUrlPath.trim().isEmpty()) {
                this.log.config("The URL path is not defined. Default value used: " + DEFAULT_HTTP_PATH);
                this.httpUrlPath = DEFAULT_HTTP_PATH;
            } else {
                this.log.config("URL path: " + this.httpUrlPath);
            }
        } else {
            this.log.config("No extra configuration defined for the WS API. Default values used");
            this.httpUrlPath = DEFAULT_HTTP_PATH;
            this.httpUrlPort = DEFAULT_HTTP_LISTENING_PORT;
        }

        try {
            this.servers = new HashMap<String, Server>();
            this.exposeServices();
        } finally {
            this.log.end();
        }
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        // TODO : delete all the web services...
        this.log.call();
    }

    private void exposeServices() {
        // expose all the services as web services...
        for (final KernelWebService ws : this.webServices.values()) {
            final Object[] itfs = ws.getComponent().getFcInterfaces();
            for (final Object object : itfs) {
                boolean registered = false;
                final Class<?>[] cs = object.getClass().getInterfaces();
                for (final Class<?> class1 : cs) {
                    final boolean isWs = WebServiceHelper.hasWebServiceAnnotation(class1);
                    if (isWs && !registered) {
                        registered = this.exposeAsWebService(object);
                    }
                }
            }
        }
    }

    protected boolean exposeAsWebService(Object o) {
        boolean result = true;

        JaxWsServerFactoryBean sf = new JaxWsServerFactoryBean();
        sf.setDataBinding(new JAXBDataBinding());
        sf.setServiceBean(o);

        String serviceName = null;
        Class<?> wsClass = WebServiceHelper.getWebServiceClass(o.getClass());

        if (wsClass != null) {
            serviceName = WebServiceHelper.getWebServiceName(wsClass);
            if ((serviceName == null) || (this.servers.get(serviceName) != null)) {
                serviceName = o.getClass().getSimpleName();
            }
            sf.setServiceClass(wsClass);
        }

        String url = this.getURL(serviceName);
        String wsdl = url + "?wsdl";
        if (this.log.isDebugEnabled()) {
            this.log.debug("Service URL is " + url);
            this.log.debug("Service WSDL is " + wsdl);
        }
        sf.setAddress(url);
        Server srv = sf.create();

        // TODO : Store the address and why not displaying the services list
        // It seems that services list is only provided when using CXFServlet,
        // need to have a look to that
        // JettyHTTPServerEngineFactory jetty =
        // svrFactory.getBus().getExtension(JettyHTTPServerEngineFactory.class);
        // JettyHTTPServerEngine engine =
        // jetty.retrieveJettyHTTPServerEngine(8084);
        // Server server = engine.getServer();

        this.servers.put(serviceName, srv);
        if (this.log.isInfoEnabled()) {
            this.log.info("Kernel Web service '" + serviceName
                    + "' has been added and is available at '" + url + "'");
        }

        return result;
    }

    protected String getURL(final String serviceName) {

        final ContainerConfiguration cc = this.configurationService.getContainerConfiguration();
        return "http://" + cc.getHost() + ":" + this.httpUrlPort + "/" + this.httpUrlPath + "/" + serviceName;
    }

    /**
     * {@inheritDoc}
     */
    public List<String> getServicesURL() {
        List<String> result = new ArrayList<String>();
        for (Server server : this.servers.values()) {
            result.add(server.getEndpoint().getEndpointInfo().getAddress());
        }
        return result;
    }

    public List<String> getServiceNames() {
        return Arrays.asList(this.servers.keySet()
                .toArray(new String[this.servers.keySet().size()]));
    }

    /**
     * {@inheritDoc}
     */
    public void removeService(String name) throws WebServiceException {
        Server server = this.servers.get(name);
        if (server != null) {
            try {
                server.stop();
            } catch (Throwable e) {
                final String message = "Web Service " + name + " can not be removed";
                if (this.log.isWarnEnabled()) {
                    this.log.warning(message, e);
                }
                throw new WebServiceException(message, e);
            }
            this.servers.remove(name);
        }
    }
}
