/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.extension.artifactrepository.api.Artifact;
import org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.DeploymentService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.microkernel.api.jbi.management.AtomicDeploymentService;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "webservice", signature = DeploymentService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class DeploymentServiceImpl implements DeploymentService, KernelWebService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.DeploymentService";

    @Controller
    private Component component;

    @Requires(name = "deployment")
    private AtomicDeploymentService deploymentService;

    @Requires(name = "artifactrepository")
    private ArtifactRepositoryService artifactRepositoryService;
    
    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    @Override
    public boolean deploy(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : deploy with params saId = " + saId);
        }
        boolean result = true;

        List<Artifact> sas = this.artifactRepositoryService.getServiceAssemblies();
        Iterator<Artifact> iter = sas.iterator();
        boolean found = false;
        Artifact artifact = null;
        while (iter.hasNext() && !found) {
            Artifact a = iter.next();
            if ((a.getName() != null) && a.getName().equals(saId)) {
                artifact = a;
                found = true;
            }
        }

        if (artifact == null) {
            throw new PEtALSWebServiceException("Can not find artifact " + saId
                    + " in the repository");
        }

        try {
            result = this.deploymentService.deploy(artifact.getFile().toURI().toURL());
        } catch (Exception e) {
            String message = "Can not deploy the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    @Override
    public boolean shutdown(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : shutdown with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.shutdown(saId);
        } catch (PetalsException e) {
            String message = "Can not shutdown the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    @Override
    public boolean start(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : start with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.start(saId);
        } catch (PetalsException e) {
            String message = "Can not start the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    @Override
    public boolean stop(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : stop with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.stop(saId);
        } catch (PetalsException e) {
            String message = "Can not stop the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    @Override
    public boolean undeploy(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : undeploy with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.undeploy(saId);
        } catch (PetalsException e) {
            String message = "Can not undeploy the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    @Override
    public Component getComponent() {
        return this.component;
    }

}
