/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Contingency;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.dragon.connection.api.service.EeType;
import org.ow2.dragon.connection.api.service.ExecutionEnvironmentManager;
import org.ow2.dragon.connection.api.service.FedPattern;
import org.ow2.dragon.connection.api.service.Node;
import org.ow2.dragon.connection.api.to.Endpoint;
import org.ow2.dragon.connection.api.to.EnvironmentFederation;
import org.ow2.dragon.connection.api.to.ExecutionEnvironment;
import org.ow2.dragon.connection.api.to.HashMapEntryType;
import org.ow2.dragon.connection.api.to.Processor;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlReaders;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlWriters;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;
import com.ebmwebsourcing.easycommons.pooling.PoolException;
import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "webservice", signature = ExecutionEnvironmentManager.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class DragonConnectionServiceImpl implements KernelWebService, ExecutionEnvironmentManager {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.DragonConnectionService";

    @Controller
    private Component component;

    @Requires(name = "endpoint")
    private EndpointRegistry endpointRegistry;

    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    @Requires(name = "topology", contingency = Contingency.OPTIONAL)
    private TopologyService topologyService;

    @Requires(name = "adminService")
    private AdminService adminService;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    public List<ExecutionEnvironment> getFederationMembers(final String federationName) {
        final List<ExecutionEnvironment> result = new ArrayList<ExecutionEnvironment>();

        // FIXME : Federation members are the containers from the same
        // subdomain?
        final Set<ContainerConfiguration> containers = this.getContainersConfiguration();
        for (final ContainerConfiguration containerConfiguration : containers) {
            if (this.isInFederation(containerConfiguration, federationName)) {
                result.add(this.getExecutionEnvironment(containerConfiguration));
            }
        }
        return result;
    }

    public List<Endpoint> getHostedEndpointsOnExecEnv(final String execEnvName) {
        return this.getEndpointsForContainer(execEnvName);
    }

    public List<Endpoint> getHostedEndpointsOnProcessor(final String processorName) {
        // TODO : Get all the execution environments from the processor name...
        final List<Endpoint> result = new ArrayList<Endpoint>();

        final Set<ContainerConfiguration> containers = this
                .getContainersForProcessor(processorName);
        for (final ContainerConfiguration container : containers) {
            result.addAll(this.getEndpointsForContainer(container.getName()));
        }

        return result;
    }

    public List<ExecutionEnvironment> getManagedExecutionEnvironments() {
        final List<ExecutionEnvironment> result = new ArrayList<ExecutionEnvironment>();

        // get all the containers configuration
        final Set<ContainerConfiguration> set = this.getContainersConfiguration();

        // fill the exec env from the containers configuration
        for (final ContainerConfiguration containerConfiguration : set) {
            result.add(this.getExecutionEnvironment(containerConfiguration));
        }
        return result;
    }

    public org.ow2.dragon.connection.api.to.ExecutionEnvironmentManager getProperties() {
        final org.ow2.dragon.connection.api.to.ExecutionEnvironmentManager manager = new org.ow2.dragon.connection.api.to.ExecutionEnvironmentManager();
        manager.setName("Manager@PEtALS-ESB:"
                + this.configurationService.getContainerConfiguration().getName());
        return manager;
    }

    /**
     * Fill an Execution environment from the container configuration
     * 
     * @param containerConfiguration
     * @return
     */
    private ExecutionEnvironment getExecutionEnvironment(
            final ContainerConfiguration containerConfiguration) {
        final EnvironmentFederation fed = new EnvironmentFederation();
        fed.setName(this.configurationService.getDomainConfiguration().getName());

        /*
         * Initialize <ExecutionEnvironment> and its uninitialized
         * attributes automatically (avoid NPE)
         */
        final ExecutionEnvironment env = new ExecutionEnvironment();
        env.setEndpoints(new Node.Endpoints());
        env.setProperties(new Node.Properties());
        
        env.setIpv4Address(containerConfiguration.getHost());
        env.setEnvType(EeType.ESB);
        env.setType("PEtALS");
        env.setVersion(this.adminService.getSystemInfo());
        env.setName(containerConfiguration.getName());
        if (this.isStandalone()) {
            env.setRoleInFederation("standalone");
        } else {
            // TODO : The role depends on the topology, a node will be a
            // member of a domain, a node parent, a routing node or
            // whatever... Will come in PEtALS v3.
            env.setRoleInFederation("peer");
        }

        // FIXME : Standalone and distributed are not the same...
        fed.setPattern(FedPattern.DISTRIBUTED);
        env.setParentFederation(fed);

        final List<Endpoint> endpoints = this.getEndpointsForContainer(containerConfiguration
                .getName());
        
        env.getEndpoints().getEndpoint().addAll(endpoints);      
        
        env.setHostProcessor(this.getProcessor(containerConfiguration));
        env.getProperties().getProperty().addAll(this.getProperties(containerConfiguration));
        return env;
    }

    /**
     * @return
     */
    private Set<ContainerConfiguration> getContainersConfiguration() {
        Set<ContainerConfiguration> set = null;

        if (this.isStandalone()) {
            final ContainerConfiguration cc = this.configurationService.getContainerConfiguration();
            set = new HashSet<ContainerConfiguration>();
            set.add(cc);
        } else {
            set = this.topologyService.getContainersConfiguration(null);
        }

        return set;
    }

    /**
     * Get all the endpoints for the given container
     * 
     * @param containerName
     * @return
     */
    private List<Endpoint> getEndpointsForContainer(final String containerName) {
        final List<Endpoint> result = new ArrayList<Endpoint>();
        List<PetalsServiceEndpoint> endpoints;
        try {
            endpoints = this.endpointRegistry.getEndpoints();
        } catch (RegistryException e) {
            return result;
        }
        for (final PetalsServiceEndpoint ep : endpoints) {
            if (containerName.equals(ep.getLocation().getContainerName())) {
                final Endpoint endpoint = new Endpoint();
                final QName endpointName = QName.valueOf(ep.getEndpointName());
                endpoint.setName(endpointName);
                try {
                    WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = null;
                    WSDL4ComplexWsdlWriter wsdl4ComplexWsdlWriter = null;
                    try {
                        wsdl4ComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
                        Description desc = wsdl4ComplexWsdlReader.read(ep.getDescription());
                        
                        Map<URI, org.w3c.dom.Document> imports = desc.deleteImportedDocumentsInWsdl();

                        wsdl4ComplexWsdlWriter = WSDL4ComplexWsdlWriters.takeWSDL4ComplexWsdlWriter();
                        String rootWSDL = wsdl4ComplexWsdlWriter.writeWSDL4ComplexWsdl(
                                desc);
                        
                        endpoint.setWsdlDescription(rootWSDL);

                        HashMap<String, String> wsdlDescriptionImports = new HashMap<String, String>();
                        if (imports != null) {
                            for (URI uri : imports.keySet()) {
                                try {
                                    wsdlDescriptionImports.put(uri.toString(), XMLHelper
                                            .createStringFromDOMDocument(imports.get(uri)));
                                } catch (TransformerException e) {
                                    // skipped
                                    this.log.warning(e.getMessage());
                                } catch (PoolException e) {
                                    // skipped
                                    this.log.warning(e.getMessage());
                                }
                            }
                        }
                        
                        final Iterator<Entry<String, String>> it = wsdlDescriptionImports.entrySet().iterator();
                        
                        while(it.hasNext()) {
                            final Entry<String, String> entry = it.next();
                            final HashMapEntryType hashMapEntryType = new HashMapEntryType();
                            hashMapEntryType.setKey(entry.getKey());
                            hashMapEntryType.setValue(entry.getValue());
                            endpoint.getWsdlDescriptionImports().getEntry().add(hashMapEntryType);
                        }
                        
                        result.add(endpoint);
                    } finally {
                        if (wsdl4ComplexWsdlReader != null) {
                            WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(wsdl4ComplexWsdlReader);
                        }
                        if (wsdl4ComplexWsdlReader != null) {
                            WSDL4ComplexWsdlWriters.releaseWSDL4ComplexWsdlWriter(wsdl4ComplexWsdlWriter);
                        }
                    }
                } catch (WSDL4ComplexWsdlException e) {
                    this.log.warning(e.getMessage());
                } catch (URISyntaxException e) {
                    this.log.warning(e.getMessage());
                }
            }
        }
        return result;
    }

    /**
     * FIXME : how to get processor information for a PEtALS node ?
     * 
     * @param configuration
     * @return
     */
    private Processor getProcessor(final ContainerConfiguration configuration) {
        final Processor processor = new Processor();
        processor.setIpv4Address(configuration.getHost());
        processor.setName("ProcessorName-" + configuration.getName());
        processor.setType("Unknown");
        return processor;
    }

    /**
     * Get the container properties
     * 
     * @param configuration
     * @return
     */
    private List<String> getProperties(final ContainerConfiguration configuration) {
        final List<String> result = new ArrayList<String>();
        result.add("Description=" + configuration.getDescription());
        result.add("State=" + configuration.getState());
        return result;
    }

    /**
     * Get all the containers which are hosted on given the processor
     * 
     * @param processorName
     * @return
     */
    private Set<ContainerConfiguration> getContainersForProcessor(final String processorName) {
        final Set<ContainerConfiguration> result = new HashSet<ContainerConfiguration>();
        final Set<ContainerConfiguration> containers = this.getContainersConfiguration();
        for (final ContainerConfiguration containerConfiguration : containers) {
            // FIXME : For now the processor is the container...
            if (processorName.equals(containerConfiguration.getName())) {
                result.add(containerConfiguration);
            }
        }
        return result;
    }

    /**
     * Test if the container belongs to a federation. FIXME : Define a
     * federation in PEtALS.
     * 
     * @param configuration
     * @param federationName
     * @return
     */
    private boolean isInFederation(final ContainerConfiguration configuration,
            final String federationName) {
        return federationName.equals(configuration.getSubdomainName());
    }

    /**
     * 
     * @return
     */
    private boolean isStandalone() {
        return this.topologyService == null;
    }
}
