/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.extensions.Controller;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.AddContainerException;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.extension.wsapi.service.adapters.TopologyConfigurationToTopologyInformation;
import org.ow2.petals.extension.wsapi.service.adapters.TopologyInformationToTopologyConfiguration;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.TopologyService;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation;
import org.ow2.petals.kernel.ws.api.to.DomainInformation;
import org.ow2.petals.kernel.ws.api.to.JmxServiceInformation;
import org.ow2.petals.kernel.ws.api.to.JndiInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation.SubdomainMode;
import org.ow2.petals.kernel.ws.api.to.TransportServiceInformation;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.JndiConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides = {
        @Interface(name = "webservice", signature = TopologyService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class TopologyServiceImpl implements TopologyService, KernelWebService {

    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.TopologyService";

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Controller
    private Component component;

    @Requires(name = "topology")
    private org.ow2.petals.microkernel.api.communication.topology.TopologyService topologyService;

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public JndiInformation getJndiInformation() throws PEtALSWebServiceException {
        JndiInformation result = null;
        DomainConfiguration domainConfiguration = this.topologyService.getDomainConfiguration();

        if (domainConfiguration == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the domain configuration.");
            throw new PEtALSWebServiceException(errorSB.toString());
        }
        JndiConfiguration jndiConfiguration = domainConfiguration.getJndiConfiguration();

        if (jndiConfiguration == null) {
            return result;
        }

        if (StringHelper.isNullOrEmpty(jndiConfiguration.getJndiFactory())) {
            return result;
        }

        if ((jndiConfiguration.getJndiProviderUrl() == null)
                || (StringHelper.isNullOrEmpty(jndiConfiguration.getJndiProviderUrl()
                        .toASCIIString()))) {
            return result;
        }

        result = new JndiInformation();

        result.setProviderUrl(jndiConfiguration.getJndiProviderUrl().toString());
        result.setFactory(jndiConfiguration.getJndiFactory());

        if (!StringHelper.isNullOrEmpty(jndiConfiguration.getJndiSecurityPrincipal())) {
            result.setSecurityPrincipal(jndiConfiguration.getJndiSecurityPrincipal());
        }

        if (!StringHelper.isNullOrEmpty(jndiConfiguration.getJndiSecurityCredentials())) {
            result.setSecurityPrincipal(jndiConfiguration.getJndiSecurityCredentials());
        }

        result.setBatchSize(domainConfiguration.getJndiConfiguration().getJndiBatchSize());
        result.setPoolSize(domainConfiguration.getJndiConfiguration().getJndiPoolSize());

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public void addContainerInformation(final ContainerInformation addedContainer, final String subdomainName)
            throws PEtALSWebServiceException {

        if (addedContainer == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't add a null container to the topology");
            throw new PEtALSWebServiceException(errorSB.toString());
        }

        try {

            final ContainerConfiguration addedContainerConfiguration = new ContainerConfiguration();
            addedContainerConfiguration.setHost(addedContainer.getHost());
            addedContainerConfiguration.setName(addedContainer.getName());
            addedContainerConfiguration.setDescription(addedContainer.getDescription());
            addedContainerConfiguration.setUser(addedContainer.getUser());
            addedContainerConfiguration.setPassword(addedContainer.getPassword());
            addedContainerConfiguration.setSubdomainName(subdomainName);

            JmxServiceInformation jmxServiceInformation = addedContainer.getJmxService();
            if (jmxServiceInformation != null) {
                addedContainerConfiguration.setJmxRMIConnectorPort(jmxServiceInformation
                        .getRmiPort());
            } else {
                throw new PEtALSWebServiceException("Can't find remote container JMX configuration");
            }

            TransportServiceInformation transportServiceInformation = addedContainer
                    .getTransportService();
            if (transportServiceInformation != null) {
                addedContainerConfiguration.setTCPPort(transportServiceInformation.getTcpPort());
            } else {
                throw new PEtALSWebServiceException(
                        "Can't find remote container transporter configuration");
            }

            this.topologyService.addContainerConfiguration(addedContainerConfiguration);
        } catch (final AddContainerException ex) {
            throw new PEtALSWebServiceException(ex);
        } catch (final PetalsException e) {
            throw new PEtALSWebServiceException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void removeContainerInformation(final String containerName) throws PEtALSWebServiceException {

        try {
            final ContainerConfiguration removedContainerConfiguration = new ContainerConfiguration();
            removedContainerConfiguration.setName(containerName);
            this.topologyService.removeContainerConfiguration(removedContainerConfiguration);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }
    }

    /**
     * {@inheritDoc}
     */
    public List<ContainerInformation> getAllContainerInformation() throws PEtALSWebServiceException {
        Set<ContainerConfiguration> configs = this.topologyService.getContainersConfiguration(null);

        List<ContainerInformation> result = new ArrayList<ContainerInformation>();
        for (ContainerConfiguration cc : configs) {
            try {
                result.add(TopologyConfigurationToTopologyInformation
                        .containerConfigurationToContainerInformation(cc));
            } catch (PetalsException ex) {
                throw new PEtALSWebServiceException(
                        "Can't transform ContainerConfiguration to ContainerInformation");
            }
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public ContainerInformation getContainerInformation(String containerId)
            throws PEtALSWebServiceException {
        ContainerInformation result = null;
        ContainerConfiguration cc = this.topologyService.getContainerConfiguration(containerId);

        if (cc == null) {
            throw new PEtALSWebServiceException("This is not a valid container name ("
                    + containerId + " not found in topology)");
        }
        try {
            result = TopologyConfigurationToTopologyInformation
                    .containerConfigurationToContainerInformation(cc);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(
                    "Can't transform ContainerConfiguration to ContainerInformation : ", ex);
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public void addSubdomain(String name, String description, SubdomainMode subdomainMode,
            boolean networkTimeSynchronized) throws PEtALSWebServiceException {
        SubDomainConfiguration newSubdomainConfiguration = new SubDomainConfiguration();
        newSubdomainConfiguration.setName(name);
        newSubdomainConfiguration.setDescription(description);
        newSubdomainConfiguration.setNetworkTimeSynchronized(networkTimeSynchronized);

        switch (subdomainMode) {
            case DYNAMIC:
                newSubdomainConfiguration.setMode(SubDomainConfiguration.SubdomainMode.DYNAMIC);
                break;
            case STATIC:
                newSubdomainConfiguration.setMode(SubDomainConfiguration.SubdomainMode.STATIC);
                break;
            default:
                throw new PEtALSWebServiceException("Unsupported subdomain mode");
        }
        try {
            this.topologyService.addSubdomainConfiguration(newSubdomainConfiguration);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void removeSubdomain(String subdomainName) throws PEtALSWebServiceException {

        try {
            final SubDomainConfiguration subdomainToRemove = new SubDomainConfiguration();
            subdomainToRemove.setName(subdomainName);
            this.topologyService.removeSubdomainConfiguration(subdomainToRemove);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex.getMessage(), ex);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void addSubdomainWithContainers(SubdomainInformation subdomain)
            throws PEtALSWebServiceException {
        if (subdomain == null) {
            throw new PEtALSWebServiceException("No subdomain found");
        }

        String subdomainName = subdomain.getName();

        // Add the subdomain first
        String description = subdomain.getDescription();
        boolean networkTimeSynchronized = subdomain.isNetworkTimeSynchronized();
        SubdomainMode subdomainMode = subdomain.getSubdomainMode();

        this.addSubdomain(subdomainName, description, subdomainMode, networkTimeSynchronized);

        // Then add the containers (can be quite slow)
        List<ContainerInformation> containers = subdomain.getContainers();

        try {
            for (ContainerInformation container : containers) {
                this.topologyService.addContainerConfiguration(TopologyInformationToTopologyConfiguration
                        .containerInformationToContainerConfiguration(container, subdomainName));
            }
        } catch (final AddContainerException e) {
            throw new PEtALSWebServiceException(e);
        } catch (final PetalsException e) {
            throw new PEtALSWebServiceException(e);
        }
    }

    public SubdomainInformation getSubdomainByName(String subdomainName)
            throws PEtALSWebServiceException {
        SubDomainConfiguration retrievedSubdomain = null;
        SubdomainInformation subdomainInformation = null;

        if (subdomainName == null) {
            throw new PEtALSWebServiceException("No subdomain name found");
        }

        try {
            Set<SubDomainConfiguration> subdomainList = this.topologyService
                    .getSubDomainsConfiguration();
            if (subdomainList != null) {
                for (SubDomainConfiguration subdomainConfiguration : subdomainList) {
                    if (subdomainConfiguration.getName().equals(subdomainName)) {
                        retrievedSubdomain = subdomainConfiguration;
                        break;
                    }
                }
            }

            if (retrievedSubdomain == null) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't retrieve subdomain for name " + subdomainName);
            }

            // A subdomain without any containers
            subdomainInformation = TopologyConfigurationToTopologyInformation
                    .subDomainConfigurationToSubdomainInformation(retrievedSubdomain);

            Set<ContainerConfiguration> registeredContainersConfigurations = this.topologyService
                    .getContainersConfigurationsForLocalSubdomain();
            List<ContainerInformation> registeredContainersInformations = new ArrayList<ContainerInformation>();

            for (ContainerConfiguration containerConfiguration : registeredContainersConfigurations) {
                registeredContainersInformations.add(TopologyConfigurationToTopologyInformation
                        .containerConfigurationToContainerInformation(containerConfiguration));
            }

            subdomainInformation.setContainers(registeredContainersInformations);

        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }

        return subdomainInformation;
    }

    public DomainInformation getDomainInformation() throws PEtALSWebServiceException {
        DomainInformation domainInformation = new DomainInformation();

        try {
            domainInformation.setName(this.topologyService.getDomainConfiguration().getName());
            domainInformation.setDescription(this.topologyService.getDomainConfiguration()
                    .getDescription());
            domainInformation.setJndiInformation(this.getJndiInformation());

            List<SubdomainInformation> subdomainsInformations = new ArrayList<SubdomainInformation>();

            for (SubDomainConfiguration subdomainConfiguration : this.topologyService
                    .getSubDomainsConfiguration()) {
                SubdomainInformation subInfo = TopologyConfigurationToTopologyInformation
                        .subDomainConfigurationToSubdomainInformation(subdomainConfiguration);
                subdomainsInformations.add(subInfo);
            }

            domainInformation.setSubdomains(subdomainsInformations);

        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }

        return domainInformation;
    }
}
