/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service.adapters;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation;
import org.ow2.petals.kernel.ws.api.to.DomainInformation;
import org.ow2.petals.kernel.ws.api.to.JmxServiceInformation;
import org.ow2.petals.kernel.ws.api.to.JndiInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation.SubdomainMode;
import org.ow2.petals.kernel.ws.api.to.TransportServiceInformation;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.JndiConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;

/**
 * Simple adapter class that probides a way to convert **remote** topology
 * information to **local** topology configuration.
 * <br /><br />
 * <b>Caution : </b>
 * this class DOES NOT takes the elements contained within the **remote**
 * informations to convert them to **local** configuration. You have to convert
 * them manually.
 *
 * @author Charles Casadei - EBM WebSourcing
 */
public class TopologyConfigurationToTopologyInformation {

    /**
     * Adapts a ContainerConfiguration instance to a ContainerInformation one.
     * 
     * @param containerConfiguration
     *      the ContainerConfiguration instance to be adapted
     * @return
     *      the corresponding ContainerInformation instance
     * @throws PetalsException
     *      if the RegistryMode is invalid
     */
    public static ContainerInformation containerConfigurationToContainerInformation(ContainerConfiguration containerConfiguration) throws PetalsException {
        ContainerInformation containerInformation = new ContainerInformation();

        if (containerConfiguration == null) {
            return null;
        }

        containerInformation.setName(containerConfiguration.getName());
        containerInformation.setDescription(containerConfiguration.getDescription());
        containerInformation.setHost(containerConfiguration.getHost());
        containerInformation.setUser(containerConfiguration.getUser());
        containerInformation.setPassword(containerConfiguration.getPassword());

        TransportServiceInformation transportServiceInformation = new TransportServiceInformation();
        transportServiceInformation.setTcpPort(containerConfiguration.getTCPPort());
        containerInformation.setTransportService(transportServiceInformation);

        JmxServiceInformation jmxServiceInformation = new JmxServiceInformation();
        jmxServiceInformation.setRmiPort(containerConfiguration.getJmxRMIConnectorPort());
        containerInformation.setJmxService(jmxServiceInformation);

        return containerInformation;
    }

    /**
     * Adapts a SubDomainConfiguration to a SubdomainInformation one, without
     * taking care of the containers contained within the subdomain.
     *
     * @param subdomainConfiguration
     *      the SubDomainConfiguration instance to be adapted
     * @return
     *      the corresponding adapted SubdomainInformation instance
     * @throws PetalsException
     *      if the SubdomainMode is invalid
     */
    public static SubdomainInformation subDomainConfigurationToSubdomainInformation(SubDomainConfiguration subdomainConfiguration) throws PetalsException {

        SubdomainInformation subdomainInformation = new SubdomainInformation();

        if (subdomainConfiguration == null) {
            return null;
        }

        subdomainInformation.setName(subdomainConfiguration.getName());
        subdomainInformation.setDescription(subdomainConfiguration.getDescription());
        subdomainInformation.setNetworkTimeSynchronized(subdomainConfiguration.isNetworkTimeSynchronized());

        SubdomainMode subdomainMode;
        switch (subdomainConfiguration.getMode()) {
            case DYNAMIC:
                subdomainMode = SubdomainMode.DYNAMIC;
                break;
            case STATIC:
                subdomainMode = SubdomainMode.STATIC;
                break;
            default :
                throw new PetalsException("Unknown subdomain mode");
        }

        subdomainInformation.setSubdomainMode(subdomainMode);

        return subdomainInformation;
    }

    /**
     * Adapts a DomainConfiguration instance to a DomainInformation one, without 
     * taking care of the subdomains contained within the DomainConfiguration instance.
     *
     * @param domainConfiguration
     *      the DomainConfiguration instance to be adapted
     * @return
     *      the corresponding DomainInformation instance
     * @throws PetalsException
     *      if the domain mode is invalid
     */
    public static DomainInformation domainConfigurationToDomainInformation(DomainConfiguration domainConfiguration) throws PetalsException {
        DomainInformation domainInformation = new DomainInformation();

        if (domainConfiguration == null) {
            return null;
        }

        domainInformation.setName(domainConfiguration.getName());
        domainInformation.setDescription(domainConfiguration.getDescription());
        
        JndiConfiguration jndiConfiguration = domainConfiguration.getJndiConfiguration();
        if (jndiConfiguration != null) {
            JndiInformation jndiInformation = new JndiInformation();

            jndiInformation.setBatchSize(jndiConfiguration.getJndiBatchSize());
            jndiInformation.setPoolSize(jndiConfiguration.getJndiPoolSize());
            jndiInformation.setFactory(jndiConfiguration.getJndiFactory());
            jndiInformation.setProviderUrl(jndiConfiguration.getJndiProviderUrl().toString());
            jndiInformation.setSecurityPrincipal(jndiConfiguration.getJndiSecurityPrincipal());
            jndiInformation.setSecurityCredentials(jndiInformation.getSecurityCredentials());

            domainInformation.setJndiInformation(jndiInformation);
        }

        return domainInformation;
    }

}
