/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service.adapters;

import java.net.URI;
import java.net.URISyntaxException;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation;
import org.ow2.petals.kernel.ws.api.to.DomainInformation;
import org.ow2.petals.kernel.ws.api.to.JmxServiceInformation;
import org.ow2.petals.kernel.ws.api.to.JndiInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation;
import org.ow2.petals.kernel.ws.api.to.TransportServiceInformation;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.JndiConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;

/**
 * @author Charles Casadei - EBM WebSourcing
 */
public class TopologyInformationToTopologyConfiguration {

    /**
     * Adapts a ContainerInformation instance to a ContainerConfiguration one.
     *
     * @param containerInformation
     *      the ContainerInformation instance to be adapted
     * @return
     *      the corresponding ContainerConfiguration instance
     */
    public static ContainerConfiguration containerInformationToContainerConfiguration(
            final ContainerInformation containerInformation) {
        return TopologyInformationToTopologyConfiguration.containerInformationToContainerConfiguration(
                containerInformation, null);
    }

    /**
     * Adapts a ContainerInformation instance to a ContainerConfiguration one.
     * 
     * @param containerInformation
     *            the ContainerInformation instance to be adapted
     * @param subdomainName
     *            the sub-domain name of the container. Can be <code>null</code>
     * @return the corresponding ContainerConfiguration instance
     */
    public static ContainerConfiguration containerInformationToContainerConfiguration(
            final ContainerInformation containerInformation, final String subdomainName) {
        ContainerConfiguration containerConfiguration = new ContainerConfiguration();

        if (containerInformation == null) {
            return null;
        }

        containerConfiguration.setName(containerInformation.getName());
        containerConfiguration.setDescription(containerInformation.getDescription());
        containerConfiguration.setHost(containerInformation.getHost());
        containerConfiguration.setUser(containerInformation.getUser());
        containerConfiguration.setPassword(containerInformation.getPassword());

        JmxServiceInformation jmxServiceInformation = containerInformation.getJmxService();
        if (jmxServiceInformation != null)  {
            containerConfiguration.setJmxRMIConnectorPort(jmxServiceInformation.getRmiPort());
        }

        TransportServiceInformation transportServiceInformation = containerInformation.getTransportService();
        if (transportServiceInformation != null) {
            containerConfiguration.setTCPPort(transportServiceInformation.getTcpPort());
        }

        containerConfiguration.setSubdomainName(subdomainName);

        return containerConfiguration;
    }

    /**
     * Adapts a SubdomainInformation to a SubDomainConfiguration one, without
     * taking care of the containers contained within the subdomain.
     *
     * @param subdomainInformation
     *      the SubdomainInformation instance to be adapted
     * @return
     *      the corresponding adapted SubDomainConfiguration instance
     * @throws PetalsException
     *      if the SubdomainMode is invalid
     */
    public static SubDomainConfiguration subdomainInformationToSubDomainConfiguration(SubdomainInformation subdomainInformation) throws PetalsException {
        SubDomainConfiguration subdomainConfiguration = new SubDomainConfiguration();

        if (subdomainInformation == null) {
            return null;
        }

        subdomainConfiguration.setName(subdomainInformation.getName());
        subdomainConfiguration.setDescription(subdomainInformation.getDescription());
        subdomainConfiguration.setNetworkTimeSynchronized(subdomainInformation.isNetworkTimeSynchronized());

        final SubDomainConfiguration.SubdomainMode subdomainMode;
        switch(subdomainInformation.getSubdomainMode()) {
            case DYNAMIC:
                subdomainMode = SubDomainConfiguration.SubdomainMode.DYNAMIC;
                break;
            case STATIC:
                subdomainMode = SubDomainConfiguration.SubdomainMode.STATIC;
                break;
            default :
                throw new PetalsException("Unknown subdomain mode");
        }
        subdomainConfiguration.setMode(subdomainMode);


        return subdomainConfiguration;
    }

    /**
     * Adapts a DomainInformation instance to a DomainConfiguration one, without
     * taking care of the subdomains contained within the DomainInformation instance.
     *
     * @param domainInformation
     *      the DomainInformation instance to be adapted
     * @return
     *      the corresponding DomainConfiguration instance
     * @throws PetalsException
     *      if the domain mode is invalid
     */
    public static DomainConfiguration domainInformationToDomainInformation(DomainInformation domainInformation) throws PetalsException {
        DomainConfiguration domainConfiguration = new DomainConfiguration();

        if (domainInformation == null) {
            return null;
        }

        domainConfiguration.setName(domainInformation.getName());
        domainConfiguration.setDescription(domainInformation.getDescription());

        JndiInformation jndiInformation = domainInformation.getJndi();
        if (jndiInformation != null) {
            try {
                JndiConfiguration jndiConfiguration = new JndiConfiguration();
                domainConfiguration.setJndiConfiguration(jndiConfiguration);
                
                jndiConfiguration.setJndiProviderUrl(new URI(jndiInformation.getProviderUrl()));
                jndiConfiguration.setJndiFactory(jndiInformation.getFactory());

                jndiConfiguration.setJndiBatchSize(jndiInformation.getBatchSize());
                jndiConfiguration.setJndiPoolSize(jndiInformation.getPoolSize());
                
                jndiConfiguration.setJndiSecurityPrincipal(jndiInformation.getSecurityPrincipal());
                jndiConfiguration.setJndiSecurityCredentials(jndiInformation.getSecurityCredentials());
            } catch (URISyntaxException ex) {
                throw new PetalsException("No valid URI found for the JNDI provider URL", ex);
            }

        }
        
        return domainConfiguration;
    }
}
