/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi;

import static org.junit.Assert.assertEquals;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link WebServiceManagerImpl}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class WebServiceManagerImplTest {

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the container extra configuration is <code>null</code>.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the default value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_ContainerExtraConfigurationNull() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        containerConfig.setExtraConfiguration(null);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);
        
        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);
        
        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);
        
        assertEquals(WebServiceManager.DEFAULT_HTTP_LISTENING_PORT,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPort"));
        assertEquals(WebServiceManager.DEFAULT_HTTP_PATH,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPath"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP listening port is not set.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the default value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpPortNotSet() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(WebServiceManager.DEFAULT_HTTP_LISTENING_PORT,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPort"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP listening port is set to an empty value.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the default value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpPortSetEmpty() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        extraConfiguration.put(WebServiceManager.PROPERTY_NAME_HTTP_LISTENING_PORT, "");
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(WebServiceManager.DEFAULT_HTTP_LISTENING_PORT,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPort"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP listening port is set to an invalid value.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the default value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpPortSetInvalidValue() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        extraConfiguration.put(WebServiceManager.PROPERTY_NAME_HTTP_LISTENING_PORT, "unvalid-value");
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(WebServiceManager.DEFAULT_HTTP_LISTENING_PORT,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPort"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP listening port is set to a valid value.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the provided value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpPortSetValidValue() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        final int expectedValue = 12345;
        extraConfiguration.put(WebServiceManager.PROPERTY_NAME_HTTP_LISTENING_PORT, Integer.toString(expectedValue));
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(expectedValue,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPort"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP URL path port is not set.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the default value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpUrlPathNotSet() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(WebServiceManager.DEFAULT_HTTP_PATH,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPath"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP URL path is set to an empty value.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the default value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpUrlPathSetEmpty() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        extraConfiguration.put(WebServiceManager.PROPERTY_NAME_HTTP_PATH, "");
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(WebServiceManager.DEFAULT_HTTP_PATH,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPath"));

    }

    /**
     * <p>
     * Check the start of {@link WebServiceManagerImpl} when the HTTP URL path is set to a valid value.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the provided value is used.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStart_HttpUrlPathSetValidValue() throws InvocationTargetException {

        final ContainerConfiguration containerConfig = new ContainerConfiguration();
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        final String expectedValue = "my-context/my-path-in-my-context";
        extraConfiguration.put(WebServiceManager.PROPERTY_NAME_HTTP_PATH, expectedValue);
        containerConfig.setExtraConfiguration(extraConfiguration);
        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfig).anyTimes();
        EasyMock.replay(cfgServiceMock);

        final WebServiceManagerImpl wsmImpl = new WebServiceManagerImpl();
        ReflectionHelper.setPrivateField(wsmImpl, "configurationService", cfgServiceMock);

        ReflectionHelper.invokePrivateMethod(wsmImpl, "start", null, null);

        assertEquals(expectedValue,
                ReflectionHelper.getPrivateFieldValue(WebServiceManagerImpl.class, wsmImpl, "httpUrlPath"));

    }

}
